/*
 * CPZipStripper 2.0
 * Written by Decebal Mihailescu [http://www.codeproject.com/script/articles/list_articles.asp?userid=634640]
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Diagnostics;

namespace CPZipStripper
{
    public partial class Config : Form
    {
        public Config()
        {
            InitializeComponent();
        }

        public const string _maskTable = "maskRow";
        public const string _maskField = "maskField";
        public const string _FolderTable = "FolderRow";
        public const string _PathField = "PathField";
        public const string _FileTable = "FileRow";
        public const string _FileField = "FileField";
        private bool _bLoading = false;
        private void Config_Load(object sender, EventArgs e)
        {
            dataGridMasks.DataSource = _dataSet.Tables[_maskTable];
            dataGridMasks.Columns[_maskField].HeaderText = "mask(accepts wild chars '*' and '?')";
            dataGridMasks.Columns[_maskField].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;

            dataGridFolders.DataSource = _dataSet.Tables[_FolderTable];
            dataGridFolders.Columns[_PathField].HeaderText = "Folders";
            dataGridFolders.Columns[_PathField].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;

            dataGridFiles.DataSource = _dataSet.Tables[_FileTable];
            dataGridFiles.Columns[_FileField].HeaderText = "Files";
            dataGridFiles.Columns[_FileField].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            _bLoading = true;
            comboBehaviour.DataSource = Enum.GetNames(typeof(Behaviour));
            _bLoading = false;
            comboBehaviour.SelectedItem = MainForm._behaviour.ToString();//Behaviour.NoStrip.ToString();

        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            
            try
            {
                XmlDocument xd = new XmlDocument();
                //get the original
                xd.Load(MainForm.cfgxmlpath);
                //save nodes not part of the dataset
                XmlNode lastFolderpath = xd["configuration"]["LastUsedFolder"];
                XmlNode behaviour = xd["configuration"]["Behaviour"];    

                //write the masks
                _dataSet.WriteXml(MainForm.cfgxmlpath);

                //restore the old saved nodes
                xd.Load(MainForm.cfgxmlpath);
                if (lastFolderpath != null)
                    xd.DocumentElement.AppendChild(lastFolderpath);
                if (behaviour != null)
                {
                    behaviour.Attributes["value"].InnerText = MainForm._behaviour.ToString();
                    xd.DocumentElement.AppendChild(behaviour);
                }
                xd.Save(MainForm.cfgxmlpath);
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "xml error", MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }

            this.Close();
        }

        private void Config_FormClosed(object sender, FormClosedEventArgs e)
        {
            MainForm prt = this.Owner as MainForm;
            prt.ReadConfig();
        }

        private void btnConfig_Click(object sender, EventArgs e)
        {
            ProcessStartInfo psi = new ProcessStartInfo();

            try
            {
                psi.FileName = MainForm.cfgxmlpath;
                psi.Verb = "Edit";
                psi.UseShellExecute = true;
                Process.Start(psi);
            }
            catch (Exception)
            {
                MessageBox.Show(this,
                    "Your system does not have a specified XML editor.",
                    "Error : XML editor missing!",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void comboBehaviour_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_bLoading)
                return; ;
            MainForm._behaviour = (Behaviour)Enum.Parse(typeof(Behaviour),
                comboBehaviour.SelectedValue.ToString(),true);
        }
    }
}